@@ -6,14 +6,16 @@ import sys
 import copy
 import time
 import signal
+import StringIO
 import hashlib
 import subprocess
 import ConfigParser
 from optparse import OptionParser, OptionGroup
 
 from PIL import Image as PImage
+from PIL import PngImagePlugin
 
-__version__ = '0.2.7'
+__version__ = '0.2.8'
 
 
 PADDING_REGEXP = re.compile("^(\d+-?){,3}\d+$")
@@ -33,9 +35,11 @@ DEFAULT_SETTINGS = {
     'algorithm': 'square',
     'ordering': 'maxside',
     'namespace': 'sprite',
+    'sprite_namespace': '%(sprite)s',
     'crop': False,
     'url': '',
     'less': False,
+    'force': False,
     'optipng': False,
     'html': False,
     'ignore_filename_paddings': False,
@@ -46,7 +50,11 @@ DEFAULT_SETTINGS = {
     'optipngpath': 'optipng',
     'optipng': False,
     'project': False,
+    'recursive': False,
+    'follow_links': False,
     'quiet': False,
+    'no_css': False,
+    'no_img': False,
     'cachebuster': False,
     'cachebuster-filename': False,
     'global_template':
@@ -151,7 +159,8 @@ class cached_property(object):
 
 def round_up(value):
     int_value = int(value)
-    return int_value + 1 if value != int_value else int_value
+    diff = 1 if int_value > 0 else -1
+    return int_value + diff if value != int_value else int_value
 
 
 def nearest_fration(value):
@@ -371,7 +380,7 @@ ALGORITHMS = {'square': SquareAlgorithm,
 
 class Image(object):
 
-    def __init__(self, name, sprite):
+    def __init__(self, name, sprite, path=None):
         """Image constructor
 
         :param name: Image name.
@@ -385,66 +394,83 @@ class Image(object):
         pseudo = set(self.filename.split('_')).intersection(PSEUDO_CLASSES)
         self.pseudo = ':%s' % list(pseudo)[-1] if pseudo else ''
 
-        image_path = os.path.join(sprite.path, name)
-        image_file = open(image_path, "rb")
+        self.path = path or os.path.join(sprite.path, name)
 
+        with open(self.path, "rb") as image_file:
+            self._data = image_file.read()
+
+    @cached_property
+    def image(self):
+        """Return a Pil representation of this image """
+        io = StringIO.StringIO(self._data)
         try:
-            source_image = PImage.open(image_file)
-            source_image.load()
-            self.image = PImage.new('RGBA', source_image.size, (0, 0, 0, 0))
+            source_image = PImage.open(io)
+            img = PImage.new('RGBA', source_image.size, (0, 0, 0, 0))
 
             if source_image.mode == 'L':
                 alpha = source_image.split()[0]
                 transparency = source_image.info.get('transparency')
                 mask = PImage.eval(alpha, lambda a: 0 if a == transparency else 255)
-                self.image.paste(source_image, (0, 0), mask=mask)
+                img.paste(source_image, (0, 0), mask=mask)
             else:
-                self.image.paste(source_image, (0, 0))
-
+                img.paste(source_image, (0, 0))
         except IOError, e:
             raise PILUnavailableError(e.args[0].split()[1])
+        finally:
+            io.close()
 
-        image_file.close()
+        # Crop the image searching for the smallest possible bounding box
+        # without losing any non-transparent pixel.
+        # This crop is only used if the crop flag is set in the config.
 
         if self.sprite.config.crop:
-            self._crop_image()
+            width, height = img.size
+            maxx = maxy = 0
+            minx = miny = sys.maxint
 
-        self.width, self.height = self.image.size
+            for x in xrange(width):
+                for y in xrange(height):
+                    if y > miny and y < maxy and maxx == x:
+                        continue
+                    if img.getpixel((x, y)) != TRANSPARENT:
+                        if x < minx:
+                            minx = x
+                        if x > maxx:
+                            maxx = x
+                        if y < miny:
+                            miny = y
+                        if y > maxy:
+                            maxy = y
+            img = img.crop((minx, miny, maxx + 1, maxy + 1))
+
+        return img
+
+    @cached_property
+    def width(self):
+        """Return Image width"""
+        return self.image.size[0]
+
+    @cached_property
+    def height(self):
+        """Return Image height"""
+        return self.image.size[1]
+
+    @cached_property
+    def absolute_width(self):
+        """Return the total width of the image taking count of the margin,
+        padding and ratio."""
         margin = int(self.sprite.config.margin)
-
-        # Both absolute_width and absolute_height are the real absolute space
-        # that we need to allocate into the sprite for this image.
-        # It's calculated using the width/height, padding and margin
-        # of this image multiplied by the max_ratio.
-        self.absolute_width = round_up(self.width +
+        return round_up(self.width +
                 (self.horizontal_padding + 2 * margin) * self.sprite.max_ratio)
-        self.absolute_height = round_up(self.height +
-                (self.vertical_padding + 2 * margin) * self.sprite.max_ratio)
 
-    def _crop_image(self):
-        """Crop the image searching for the smallest possible bounding box
-        without losing any non-transparent pixel.
-
-        This crop is only used if the crop flag is set in the config.
+    @cached_property
+    def absolute_height(self):
+        """Return the total height of the image taking count of the margin,
+        padding and ratio.
         """
-        width, height = self.image.size
-        maxx = maxy = 0
-        minx = miny = sys.maxint
-
-        for x in xrange(width):
-            for y in xrange(height):
-                if y > miny and y < maxy and maxx == x:
-                    continue
-                if self.image.getpixel((x, y)) != TRANSPARENT:
-                    if x < minx:
-                        minx = x
-                    if x > maxx:
-                        maxx = x
-                    if y < miny:
-                        miny = y
-                    if y > maxy:
-                        maxy = y
-        self.image = self.image.crop((minx, miny, maxx + 1, maxy + 1))
+        margin = int(self.sprite.config.margin)
+        return round_up(self.height +
+                (self.vertical_padding + 2 * margin) * self.sprite.max_ratio)
 
     def _generate_padding(self, padding):
         """Return a 4-elements list with the desired padding.
@@ -473,13 +499,12 @@ class Image(object):
 
         This CSS class name will have the following format:
 
-        ``.[namespace]-[sprite_name]-[image_name]{ ... }``
+        ``.[namespace]-[sprite-namespace]-[image_name]{ ... }``
 
         The image_name will only contain alphanumeric characters,
         ``-`` and ``_``. The default namespace is ``sprite``, but it could
         be overridden using the ``--namespace`` optional argument.
 
-
         * ``animals/cat.png`` will be ``.sprite-animals-cat``
         * ``animals/cow_20.png`` will be ``.sprite-animals-cow``
         * ``animals/cat_hover.png`` will be ``.sprite-animals-cat:hover``
@@ -505,24 +530,37 @@ class Image(object):
         # Clean filename
         name = re.sub(r'[^\w\-_]', '', name)
 
-        # Customize the name if necessary
         separator = self.sprite.manager.config.separator
-        if separator == CAMELCASE_SEPARATOR:
-            separator = ''
-            if self.sprite.namespace:
-                name = name[:1].capitalize() + name[1:]
 
         # Add pseudo-class information
         name = '%s%s' % (name, self.pseudo)
 
-        return separator.join([self.sprite.namespace, name])
+        # Create the minimal namespace
+        namespace = [name]
+
+        # Add sprite namespace if required
+        if self.sprite.manager.config.sprite_namespace:
+            sprite_name = re.sub(r'[^\w\-_]', '', self.sprite.name)
+            namespace.insert(0, self.sprite.manager.config.sprite_namespace % {'sprite': sprite_name})
+
+        # Add global namespace if required
+        if self.sprite.manager.config.namespace:
+            namespace.insert(0, self.sprite.manager.config.namespace)
+
+        # Handle CamelCase separator
+        if separator == CAMELCASE_SEPARATOR:
+            namespace = [n.lower().capitalize() if i > 0 else n.lower() \
+                                            for i, n in enumerate(namespace)]
+            separator = ''
+
+        return separator.join(namespace)
 
     @cached_property
     def _padding_info(self):
         """Return the padding information from the filename."""
         for block in self.filename.split('_')[:0:-1]:
             if PADDING_REGEXP.match(block):
-                return  block.split('-')
+                return block.split('-')
         return []
 
     @cached_property
@@ -544,10 +582,12 @@ class Image(object):
 
     @cached_property
     def horizontal_padding(self):
+        """Return the horizontal padding for this image."""
         return self.padding[1] + self.padding[3]
 
     @cached_property
     def vertical_padding(self):
+        """Return the vertical padding for this image."""
         return self.padding[0] + self.padding[2]
 
     def __lt__(self, img):
@@ -585,7 +625,7 @@ class Sprite(object):
         self.manager = manager
         self.images = []
         self.path = path
-        self.cachebuster_hash = ''
+        self._processed = False
 
         self.config = manager.config.extend(get_file_config(self.path))
 
@@ -603,12 +643,29 @@ class Sprite(object):
         # Create a sorted list of ratios
         self.ratios = sorted(self.ratios)
 
-        self.process()
+        # Locate images
+        self.images = self._locate_images()
+
+    def validate(self):
+        """Validate this sprite cheking that all images will have different
+        CCS class names.
+        """
+        class_names = [i.class_name for i in self.images]
+        if len(set(class_names)) != len(self.images):
+            dup = [i for i in self.images if class_names.count(i.class_name) > 1]
+            raise MultipleImagesWithSameNameError(dup)
+
+        for image in self.images:
+            self.manager.log("\t %s => .%s" % (image.name, image.class_name))
+
+        return True
 
     def process(self):
         """Process a sprite path searching for all the images and then
         allocate all of them in the most appropriate position.
         """
+        if self._processed:
+            return
 
         algorithm = ALGORITHMS.get(self.config.algorithm)
 
@@ -616,9 +673,9 @@ class Sprite(object):
             raise InvalidImageAlgorithmError(self.config.algorithm)
 
         self.algorithm = algorithm()
-        self.images = self._locate_images()
-
+        self.images = sorted(self.images, reverse=self.config.ordering[0] != '-')
         self.algorithm.process(self)
+        self._processed = True
 
     def _locate_images(self):
         """Return all valid images within a folder.
@@ -635,23 +692,19 @@ class Sprite(object):
         extensions = '|'.join(VALID_IMAGE_EXTENSIONS)
         extension_re = re.compile('.+\.(%s)$' % extensions, re.IGNORECASE)
         files = sorted(os.listdir(self.path))
-        images = [Image(n, sprite=self) for n in files if \
-                                    not n.startswith('.') and \
-                                    extension_re.match(n)]
 
-        if not len(images):
+        images = []
+        for root, dirs, files in os.walk(self.path, followlinks=self.config.follow_links):
+            for f in sorted(files):
+                if not f.startswith('.') and extension_re.match(f):
+                    images.append(Image(f, path=os.path.join(root, f), sprite=self))
+            if not self.config.recursive:
+                break
+
+        if not images:
             raise SourceImagesNotFoundError()
 
-        # Check if there are duplicate class names
-        class_names = [i.class_name for i in images]
-        if len(set(class_names)) != len(images):
-            dup = [i for i in images if class_names.count(i.class_name) > 1]
-            raise MultipleImagesWithSameNameError(dup)
-
-        for image in images:
-            self.manager.log("\t %s => .%s" % (image.name, image.class_name))
-
-        return sorted(images, reverse=self.config.ordering[0] != '-')
+        return images
 
     @cached_property
     def canvas_size(self):
@@ -668,8 +721,36 @@ class Sprite(object):
 
     def save_image(self):
         """Create the image file for this sprite."""
+
+        if self.config.no_img:
+            return
+
+        # Check if we need to create any sprite.
+        ratios_to_process = []
+
+        for ratio in self.ratios:
+            sprite_image_path = self.image_path(ratio)
+            try:
+                assert not self.config.force
+                existing_sprite = PImage.open(sprite_image_path)
+                assert existing_sprite.info['Software'] == 'glue-%s' % __version__
+                assert existing_sprite.info['Comment'] == self.hash
+                already_created = True
+            except Exception:
+                already_created = False
+
+            if not already_created:
+                ratios_to_process.append(ratio)
+
+        if not ratios_to_process:
+            self.manager.log("Already exists '%s' image file..." % self.name)
+            return
+
         self.manager.log("Creating '%s' image file..." % self.name)
 
+        # Process the sprite if necessary.
+        self.process()
+
         # Create the sprite canvas
         width, height = self.canvas_size
         canvas = PImage.new('RGBA', (width, height), (0, 0, 0, 0))
@@ -681,12 +762,12 @@ class Sprite(object):
                 (round_up(image.x + (image.padding[3] + margin) * self.max_ratio),
                  round_up(image.y + (image.padding[0] + margin) * self.max_ratio)))
 
-        if self.config.cachebuster or self.config.cachebuster_filename:
-            self.cachebuster_hash = hashlib.sha1(canvas.tostring()
-                                                ).hexdigest()[:6]
+        meta = PngImagePlugin.PngInfo()
+        meta.add_text('Software', 'glue-%s' % __version__)
+        meta.add_text('Comment', self.hash)
 
         # Customize how the png is going to be saved
-        kwargs = dict(optimize=True)
+        kwargs = dict(optimize=False, pnginfo=meta)
 
         if self.config.png8:
             # Get the alpha band
@@ -704,7 +785,7 @@ class Sprite(object):
             kwargs.update({'transparency': 255})
 
         # Loop all over the ratios and save one image for each one
-        for ratio in self.ratios:
+        for ratio in ratios_to_process:
 
             sprite_image_path = self.image_path(ratio)
 
@@ -736,18 +817,37 @@ class Sprite(object):
 
     def save_css(self):
         """Create the CSS or LESS file for this sprite."""
-        format = 'less' if self.config.less else 'css'
-        self.manager.log("Creating '%s' %s file..." % (self.name, format))
 
+        if self.config.no_css:
+            return
+
+        format = 'less' if self.config.less else 'css'
         output_path = self.manager.output_path('css')
         filename = '%s.%s' % (self.filename, format)
         css_filename = os.path.join(output_path, filename)
+        hash_line = '/* glue: %s hash: %s */\n' % (__version__, self.hash)
 
-        # Fix css urls on Windows
-        css_filename = '/'.join(css_filename.split('\\'))
+        # Check if the CSS file already exists and has the same hash
+        try:
+            assert not self.config.force
+            with open(css_filename, 'r') as existing_css:
+                first_line = existing_css.readline()
+                assert first_line == hash_line
+                self.manager.log("Already exists '%s' %s file..." % (self.name, format))
+                return
+        except Exception:
+            pass
+
+        self.manager.log("Creating '%s' %s file..." % (self.name, format))
+
+        # Process the sprite if necessary.
+        self.process()
 
         css_file = open(css_filename, 'w')
 
+        # Write the hash line to the file.
+        css_file.write(hash_line)
+
         # get all the class names and join them
         class_names = ',\n'.join(['.%s' % i.class_name for i in self.images \
                                                   if ':' not in i.class_name])
@@ -755,8 +855,7 @@ class Sprite(object):
         # add the global style for all the sprites for less bloat
         template = self.config.global_template.decode('unicode-escape')
         css_file.write(template % {'all_classes': class_names,
-                                   'sprite_url': self.image_url(),
-                                   'namespace': self.namespace})
+                                   'sprite_url': self.image_url()})
 
         # compile one template for each file
         margin = int(self.config.margin)
@@ -795,6 +894,7 @@ class Sprite(object):
         css_file.close()
 
     def save_html(self):
+        """Create the HTML file for this sprite."""
         self.manager.log("Creating '%s' html file..." % self.name)
 
         output_path = self.manager.output_path('css')
@@ -804,9 +904,6 @@ class Sprite(object):
         # CSS output format
         format = 'less' if self.config.less else 'css'
 
-        # Fix css urls on Windows
-        html_filename = '/'.join(html_filename.split('\\'))
-
         html_file = open(html_filename, 'w')
 
         # get all the class names and join them
@@ -822,37 +919,23 @@ class Sprite(object):
                                          'version': __version__})
         html_file.close()
 
-    @cached_property
-    def namespace(self):
-        """Return the namespace for this sprite."""
-
-        namespace = [self.name]
-        separator = self.config.separator
-
-        if self.config.namespace:
-            namespace.insert(0, self.config.namespace)
-
-        if separator == CAMELCASE_SEPARATOR:
-            namespace = [n.lower().capitalize() if i > 0 else n.lower() \
-                                            for i, n in  enumerate(namespace)]
-            separator = ''
-        return separator.join(namespace)
-
     @cached_property
     def filename(self):
         """Return the desired filename for files generated by this sprite."""
         if self.config.cachebuster_filename:
-            return '%s_%s' % (self.name, self.cachebuster_hash)
+            return '%s_%s' % (self.name, self.hash[:6])
         return self.name
 
-    def image_path(self, ratio=1):
+    def image_path(self, ratio=1, full=True):
         reference = self.__get_reference(ratio)
         """Return the output path for the image file.
-
+        If full, prepend the img output path, if not only return the filename.
         :param ratio: Ratio.
         """
-        return os.path.join(self.manager.output_path('img'),
-                            '%s%s.png' % (self.filename, reference))
+        filename = '%s%s.png' % (self.filename, reference)
+        if full:
+            return os.path.join(self.manager.output_path('img'), filename)
+        return filename
 
     def __get_reference(self, ratio):
         """ Return the reference @Nx for this ratio.
@@ -874,17 +957,46 @@ class Sprite(object):
 
         :param ratio: Ratio.
         """
-        url = os.path.relpath(self.image_path(ratio),
-                              self.manager.output_path('css'))
 
         if self.config.url:
-            url = os.path.join(self.config.url, url)
+            image_path = self.image_path(ratio, full=False)
+            url = os.path.join(self.config.url, image_path)
+        else:
+            image_path = self.image_path(ratio)
+            url = os.path.relpath(image_path, self.manager.output_path('css'))
+            url = os.path.normpath(url)
+
+        # Fix css urls on Windows
+        if os.name == 'nt':
+            url = url.replace('\\', '/')
 
         if self.config.cachebuster:
-            url = "%s?%s" % (url, self.cachebuster_hash)
+            url = "%s?%s" % (url, self.hash[:6])
 
         return url
 
+    @cached_property
+    def hash(self):
+        """ Return a hash of this sprite. In order to detect any change on
+        the source images  it use the data, order and path of each image.
+        In the same way it use this sprite settings as part of the hash.
+        """
+        hash_list = []
+        for image in sorted(self.images, key=lambda i: i.path):
+            hash_list.append(image.path)
+            hash_list.append(image._data)
+
+        for key in DEFAULT_SETTINGS:
+
+            # Ignore this settings as they don't change the result.
+            if key in ['html', 'quiet', 'force']:
+                continue
+
+            hash_list.append(key)
+            hash_list.append(str(getattr(self.config, key)))
+
+        return hashlib.sha1(''.join(hash_list)).hexdigest()[:10]
+
 
 class ConfigManager(object):
     """Manage all the available configuration.
@@ -926,19 +1038,34 @@ class ConfigManager(object):
         if name in self._cache:
             return self._cache[name]
 
-        sources = [self.priority] + self.sources
+        if '_%s' % name in dir(self):
+            value = self.__getattribute__('_%s' % name)()
+            self._cache[name] = value
+            return value
 
-        for source in sources:
-            value = source.get(name)
-            if value is not None:
-                self._cache[name] = value
-                break
-
-        if name not in self._cache:
-            self._cache[name] = self.defaults.get(name)
+        self._cache[name] = self.find(name)
 
         return self._cache[name]
 
+    @cached_property
+    def _sources(self):
+        return [self.priority] + self.sources
+
+    def find(self, name):
+        for source in self._sources:
+            value = source.get(name)
+            if value is not None:
+                return value
+        return self.defaults.get(name)
+
+    def _margin(self):
+        """Make retina sprites have at least 2px of margin to avoid noise
+        issues while resizing the sprite from 2x to 1x. If the --margin option
+        is present this setting will be ignored."""
+        if (self.find('retina') or self.find('ratios')) and int(self.find('margin')) < 2:
+            return 2
+        return int(self.find('margin'))
+
 
 class BaseManager(object):
 
@@ -962,12 +1089,32 @@ class BaseManager(object):
         :param path: Sprite path.
         :param name: Sprite name.
         """
-        self.log("Processing '%s':" % name)
         sprite = Sprite(name=name, path=path, manager=self)
         self.sprites.append(sprite)
 
+    def validate(self):
+        """Validate CSS class names collision between sprites"""
+
+        class_names = reduce(lambda x, y: x + y, [[i.class_name for i in sprite.images] for sprite in self.sprites])
+
+        if len(class_names) != len(set(class_names)):
+            dup = [[i for i in sprite.images if class_names.count(i.class_name) > 1] for sprite in self.sprites]
+            dup = reduce(lambda x, y: x + y, dup)
+            raise MultipleImagesWithSameNameError(dup)
+
+        return True
+
     def save(self):
         """Save all the sprites inside this manager."""
+
+        # Validate sprites individualy
+        for sprite in self.sprites:
+            self.log("Processing '%s':" % sprite.name)
+            sprite.validate()
+
+        # Validate collisions between sprites
+        self.validate()
+
         for sprite in self.sprites:
             sprite.save_image()
             sprite.save_css()
@@ -1035,13 +1182,21 @@ class ProjectSpriteManager(BaseManager):
         This is not the default manager. It is only used if you use
         the ``--project`` argument.
         """
-        for sprite_name in os.listdir(self.path):
+
+        for sprite_name in sorted(os.listdir(self.path)):
+
             # Only process folders
             path = os.path.join(self.path, sprite_name)
-            if os.path.isdir(path) and not sprite_name.startswith('.'):
+
+            # Ignore folders starting with '.'
+            if sprite_name.startswith('.'):
+                continue
+
+            # Ignore symlinks if necessary.
+            if self.config.follow_links or not os.path.islink(path):
                 self.process_sprite(path=path, name=sprite_name)
 
-        if not len(self.sprites):
+        if not self.sprites:
             raise NoSpritesFoldersFoundError()
 
         self.save()
@@ -1184,6 +1339,11 @@ def main():
                                  "| --css=<dir> --img=<dir>]"))
     parser.add_option("--project", action="store_true", dest="project",
             help="generate sprites for multiple folders")
+    parser.add_option("-r", "--recursive", dest="recursive", action='store_true',
+            help=("Read directories recursively and add all the "
+                  "images to the same sprite."))
+    parser.add_option("--follow-links", dest="follow_links", action='store_true',
+            help="Follow symbolic links.")
     parser.add_option("-c", "--crop", dest="crop", action='store_true',
             help="crop images removing unnecessary transparent margins")
     parser.add_option("-l", "--less", dest="less", action='store_true',
@@ -1198,6 +1358,9 @@ def main():
             help="Create sprites based on these ratios")
     parser.add_option("--retina", dest="retina", action='store_true',
             help="Shortcut for --ratios=2,1")
+    parser.add_option("-f", "--force", dest="force", action='store_true',
+            help=("force glue to create every sprite and CSS file even if "
+                  "they already exists in the output directory."))
     parser.add_option("-w", "--watch", dest="watch", default=False,
             action='store_true',
             help=("Watch the source folder for changes and rebuild "
@@ -1212,19 +1375,27 @@ def main():
             help="output directory for img files")
     group.add_option("--html", dest="html", action="store_true",
             help="generate test html file using the sprite image and CSS.")
+    group.add_option("--no-css", dest="no_css", action="store_true",
+            help="don't genereate CSS files.")
+    group.add_option("--no-img", dest="no_img", action="store_true",
+            help="don't genereate IMG files.")
+
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Advanced Options")
     group.add_option("-a", "--algorithm", dest="algorithm", metavar='NAME',
-            help=("allocation algorithm: square, vertical, horizontal "
-                  "(default: square)"), )
+            help=("allocation algorithm: square, vertical, horizontal, "
+                  "vertical-right, horizontal-bottom, diagonal. "
+                  "(default: square)"))
     group.add_option("--ordering", dest="ordering", metavar='NAME',
             help=("ordering criteria: maxside, width, height or "
-                  "area (default: maxside)"), )
+                  "area (default: maxside)"))
     group.add_option("--margin", dest="margin", type=int,
             help="force this margin in all images")
     group.add_option("--namespace", dest="namespace",
             help="namespace for all css classes (default: sprite)")
+    group.add_option("--sprite-namespace", dest="sprite_namespace",
+            help="namespace for all sprites (default: sprite name)")
     group.add_option("--png8", action="store_true", dest="png8",
             help="the output image format will be png8 instead of png32")
     group.add_option("--ignore-filename-paddings", action='store_true',
@@ -1312,6 +1483,7 @@ def main():
     options = options.__dict__
 
     config = ConfigManager(config, priority=options, defaults=DEFAULT_SETTINGS)
+
     manager = manager_cls(path=source, output=output, config=config)
 
     if config.optipng and not command_exists(config.optipngpath):
@@ -1328,7 +1500,8 @@ def main():
     except MultipleImagesWithSameNameError, e:
         sys.stderr.write("Error: Some images will have the same class name:\n")
         for image in e.args[0]:
-            sys.stderr.write('\t %s => .%s\n' % (image.name, image.class_name))
+            rel_path = os.path.relpath(image.path)
+            sys.stderr.write('\t%s => .%s\n' % (rel_path, image.class_name))
         sys.exit(e.error_code)
     except SourceImagesNotFoundError, e:
         sys.stderr.write("Error: No images found.\n")
@@ -1357,7 +1530,7 @@ def main():
             sys.stderr.write("Config: %s\n" % config.sources)
             sys.stderr.write("Args: %s\n" % sys.argv)
             sys.stderr.write("\n")
-            raise
+
         sys.stderr.write("Error: Unknown Error.\n")
         sys.exit(1)
 
